package com.centerm.smartpos.aidl.pinpad;

import com.centerm.smartpos.aidl.pinpad.PinPadInputPinCallback;
import com.centerm.smartpos.aidl.pinpad.PinPadDisplayCallback;
import com.centerm.smartpos.aidl.pinpad.PinPadDownloadKeyCallback;
import com.centerm.smartpos.aidl.pinpad.PinPadEncryptDataCallback;
import com.centerm.smartpos.aidl.pinpad.MacInfo;
import com.centerm.smartpos.aidl.pinpad.PinInfo;
import com.centerm.smartpos.aidl.pinpad.OnSignListener;

interface AidlPinPad {

    /**
     * 中断 Pin 输入
     *
     * @param device  设备句柄
     */
    void stopGetPin();

	/**
	 * 获取随机数
	 * @return 8字节的随机数
	 * @createtor：Administrator
	 * @date:2014-12-12 上午11:36:02
	 */
	byte[] getRandom();
	/**
	 * 发散工作密钥
	 * @param type
	 * @createtor：Administrator
	 * @date:2014-12-12 上午11:36:18
	 */
	boolean disperseWkey(byte type,in byte[] keyVal,byte tmkIndex,byte wkIndex);
	
	/**
	 * 发散主密钥，不支持国密
	 * 
	 * @createtor：Administrator
	 * @date:2014-12-12 上午11:36:34
	 */
	boolean disperseMkey(in byte[] keyVal,byte tmkIndex);
	
	/**
	 * 发散主密钥，支持国密
	 * 
	 * @createtor：Administrator
	 * @date:2014-12-12 上午11:36:34
	 */
	boolean disperseSM4Mkey(in int type,in byte[] keyVal,byte tmkIndex);
	/**
	 * 确认PIN输入，相当于按下内置密码键盘确认键，仅内置支持
	 * 
	 * @createtor：Administrator
	 * @date:2014-12-12 上午11:36:44
	 */
	void confirmGetPin();
	/**
	 * 获取PIN输入
	 * @param PinInfo PIN输入控制参数
	 * @createtor：Administrator
	 * @date:2014-12-12 上午11:37:18
	 */
	void getPin(in PinInfo pinInfo,in PinPadInputPinCallback listener);
	/**
	 * 计算MAC
	 * @param MacInfo mac计算相关参数
	 * @createtor：Administrator
	 * @date:2014-12-12 上午11:37:37
	 */
	byte[] getMac(in MacInfo macInfo);
	/**
	 * 加密数据
	 * @param encryptMode 数据加密模式分为带随机数和不带随机数
	 * @param wkIndex 工作密钥索引
	 * @createtor：Administrator
	 * @date:2014-12-12 上午11:37:56
	 */
	byte[] enCryptData(byte encryptMode,byte wkIndex,in byte[] random,in byte[] data);
	/**
	 * 密码键盘显示
	 * @param line1 第一行
	 * @param line2 第二行
	 * @param IKeyBoardDisplayCallback 操作回调
	 * @createtor：Administrator
	 * @date:2014-12-12 下午12:10:15
	 */
	boolean display(String line1,String line2);
	
	/**
	 * 获取密码键盘序列号
	 * @param index 读取的索引号，2字节大小
	 */
	String getPinPadSn(int index);
	
	/**开启密码键盘普通按键*/
	boolean openPinpadKeyboard();
	/**关闭密码键盘普通按键*/
	boolean closePinpadKeyboard();
	/**获取密码键盘普通按键状态*/
	int pinpadKeyboardState();
	/**密码键盘打开*/
	boolean open(int rate);
	/**密码键盘关闭**/
	boolean close();
	
	/**
	 * 外接非接卡卡状态
	 * @return 卡状态 0：未查询到卡，1：查询到卡
	 * @throws Exception
	 */
	byte rfCardStatus();
	
	/**
	 * 外接非接卡复位
	 * @return 成功返回复位信息，失败返回null
	 * @throws Exception
	 */
    byte[] rfCardReset();
	
	/**
	 * 外接非接卡发送指令
	 * @param apdu:发送的指令集
	 * @return 成功返回指令操作结果，失败返回null
	 * @throws Exception
	 */
	byte[] rfCardApdu(in byte[] apdu);
	
	/**
	 * 外接非接卡断开操作
	 * @return 成功返回true，失败返回false
	 * @throws Exception
	 */
	boolean rfCardHalt();
	
	/**
	 * 设置内置密码键盘提示音
	 * @return 成功返回true，失败返回false
	 * @throws Exception
	 */
	boolean setKeyVoice(in boolean isOpen);
	
	/**
	 * 下载KEK，3级密钥体系使用
	 * @param kek ：KEK明文
	 * @throws Exception
	 */
	boolean downKEY(in byte[] kek);
	
	/**
	 * 下载TMK，受KEY保护，3级密钥体系使用
	 * @param tmk：TMK明文
	 * @param checkvalue，4字节，密钥验证码,checkvalue为null则无checkvalue验证
	 * @param keyid：密钥索引
	 * @throws Exception
	 */
	boolean downTMK(in byte[] tmk,in byte[] checkvalue,in byte keyid);
	
	/**
	 * 下载tek
	 * @param mode ： 0x80  有checkValue验证，0x40没有checkValue验证
	 * @param tekId ： 目前固定为0
	 * @param key：16字节明文
	 * @param checkValue：key 3DES加密8字节0x00的结果的前4个字节
	 * @throws Exception
	 */
	boolean downTek(in byte tekId,in byte[] key,in byte[] checkvalue);
	
	/**
	 * 下载kek加密kek
	 * @param mode ： 1Byte，指示密钥下载类型 Bit7：1：手工注入；0：SM4密钥下载<br/>
	 *        Bit7：1：手工注入；<br/>
	 *        Bit6:1：不校验checkvalue<br/>
	 *            0：校验checkvalue，checkvalue的位数由bit5-bit4决定<br/>
	 *        Bit4~bit5：00：若bit6为0，checkvalue为4字节  01：若bit6为0，checkvalue为1字节
	 *            02：若bit6为0，checkvalue为2字节 03：若bit6为0，checkvalue为3字节；<br/>
	 *       Bit3~0： 00：PIKEK； 01：MAKEK; 02：TDKEK; 其他保留 <br/>
	 * @param keyId ： 指示密钥索引,加密密钥索引
	 * @param keyDownloadId ：待下载密钥索引
	 * @param key：密钥密文（原KEK加密现在的密钥）
	 * @param checkValue：4个字节校验值
	 * @throws Exception
	 */
	boolean downloadKEKEncryptKEK(in byte mode, in byte keyId,
			in byte keyDownloadId, in byte[] key, in byte[] checkvalue);
	
	/**
	 * 通过项目获取密码键盘序列号
	 * @param projectNum 项目编号
	 * @param index 读取的索引号，2字节大小
	 */
	String getPinPadSnFromProject(int projectNum , int index);
	
	/**
	 * 下载主密钥（3级密钥体系）
	 * @param tekId 指定下载的tek索引（需要先下载tek到内置密码键盘中）
	 * @param mKeyId 主密钥索引
     * @param key 密钥密文（tek密钥加密）PIN+MAC+DATA
     * @param checkValue 非空校验 - PIN CHECKVALUE ＋　MAC CHECKVALUE + DATA CHECKVALUE
     * @param len checkValue校验的长度（0/1/2/3/4)0，不校验checkValue,校验时一般使用4 
	 */
	boolean downloadKekEncryptedMKey(byte tekId, byte mKeyId, in byte[] key, in byte[] checkValue, int len);
	
	/**
	 * 下载工作密钥（3级密钥体系）
	 * @param type  指定下载的工作密钥类型(0x00:MAC加密,0x01:PIN加密，0x02:DATA加密)
	 * @param tekId 指定的tek密钥索引（需要先下载tek到内置密码键盘中）
     * @param mKeyId 指定使用的内置密码键盘的tmk索引，需要预先下载Tmk到内置密码键盘，否则返回错误
     * @param wKeyId 工作密钥索引
     * @param key 工作密钥密文（TEK加密）
	 * @param checkValue checkValue(工作密钥明文对8字节全03DES加密取4字节
	 * @param len checkValue校验的长度（0/1/2/3/4)0：不检验checkValue, 校验时候一般使用4
	 */
	boolean downloadKekEncryptedWKey(byte type, byte tekId, byte mKeyId, byte wKeyId, in byte[] key, in byte[] checkValue, int len);
	
	/**
	 * @param tag 水印：16个字符
	 * @param listener 
	 */
	void startSign(String tag, OnSignListener listener);
	
	/**
	 */
	boolean stopSign();
	
	/**
	 * 带超时时间获取PIN输入
	 * @param PinInfo PIN输入控制参数
	 * @createtor：Administrator
	 */
	void getPinWithTime(in PinInfo pinInfo,in PinPadInputPinCallback listener, int timeout);
	
	/**
	 * 获取外置密码键盘序列号
	 * @param index 读取的索引号，1字节大小
	 * @param len 读取的数据长度，index+len的最大长度不超过256
	 */
	byte[] getPinPadSnfromExternal(byte offset,byte len);
	
	/**
	 * 设置密码键盘取消模式
	 * @param mode :模式
	 * @author liqiaoli@centerm.com
	 * @time 2017/03/16 17:40
	 **/
	boolean setPinpadCancelMode(byte mode);
	
	/**
	 * 获取硬件版本号
	 * @author liqiaoli@centerm
	 * @time 2017/03/27 16:24
	 */
	byte[] getHardWareSN();
	
	/**
	 * 硬件序列号MAC计算
	 * @author wengtao@centerm.com
	 * @time 2017/04/05 14:41
	 */
	byte[] getMacForSNK(String data, String random);
}